"""
  KISC-32 Compiler | cpp_json_parser.py
  Copyright (C) 2024  Kaelan Carlos

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.

  Contact email: kelna@kelna.dev
"""

import json


# options
NAMESPACE = "lang"

INSTR_TOKS_NAME = "instr_toks"
INSTR_VARS_NAME = "instr_vars"
INSTR_DLENS_NAME = "instr_dlens"
INSTR_OPS_NAME = "instr_ops"
OPS_MAX_LENGTH = 2   # note: changes to this will still require code edits

REG_TOKS_NAME = "reg_toks"
REG_ADDRS_NAME = "reg_addrs"

FVS_TOKS_NAME = "fvs_toks"
FVS_BITS_NAME = "fvs_bits"
FVS_EDITS_NAME = "fvs_edits"


# instructions.json
with open("instructions.json", "r") as f:
  instructions: dict[str, dict] = json.load(f)
  f.close()

instr_toks: list[str] = [] # tokens
instr_vars: list[str] = [] # variabilities
instr_dlens: list[str] = [] # default lengths
instr_ops: list[str] = [] # operands

for i, (mnemonic, info) in enumerate(instructions.items()):
  # account for skips in opcodes
  while info["opcode"] != i:
    instr_toks.append("hi, this is an unused instruction. if somehow you get the tokeniser to incorrectly parse spaces, congrats. here's your special cookie: 🍪")
    instr_vars.append("false")
    instr_dlens.append("-1")
    instr_ops.append("")
    i += 1

  instr_toks.append(mnemonic)
  instr_vars.append(str(info["var"]).lower())
  instr_dlens.append(str(info["default_length"]))

  if len(info["operands"]) > OPS_MAX_LENGTH:
    raise KeyError("bruh waht")

  # yes, it's kinda repetitive but i cbb making a "smart" solution
  if len(info["operands"]) == 0:
    operands = "{' ', ' '}"
  elif len(info["operands"]) == 1:
    operands = "{'" + info["operands"][0] + "', ' '}"
  else:
    operands = "{'" + info["operands"][0] + "', '" + info["operands"][1] + "'}"
  
  instr_ops.append(operands)


# registers.json
with open("registers.json", "r") as f:
  registers: dict[str, int] = json.load(f)
  f.close()

reg_toks: list[str] = [] # tokens
reg_addrs: list[str] = [] # addresses

for mnemonic, address in registers.items():
  reg_toks.append(mnemonic)
  reg_addrs.append(str(address))


# flags_vals.json
with open("flags_vals.json", "r") as f:
  flags_vals: dict[str, dict] = json.load(f)
  f.close()

fvs_toks: list[str] = [] # tokens
fvs_bits: list[str] = [] # bits
fvs_edits: list[str] = [] # editables

for bits, info in flags_vals.items():
  fvs_toks.append(info["mnemonic"])
  fvs_bits.append(bits)
  fvs_edits.append(str(info["editable"]).lower())


# write to file
header = """/*
  KISC-32 Compiler | tokens.hpp
  Copyright (C) 2024  Kaelan Carlos

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <https://www.gnu.org/licenses/>.

  Contact email: kelna@kelna.dev
*/

#include <iostream>\n\nnamespace """ + NAMESPACE + "\n{\n"
footer = "}"

instr_toks_string = "  std::string " + INSTR_TOKS_NAME + "[] = {\n    \"" + "\",\n    \"".join(instr_toks) + "\"\n  };\n"
instr_vars_string = "  bool " + INSTR_VARS_NAME + "[] = {\n    " + ",\n    ".join(instr_vars) + "\n  };\n"
instr_dlens_string = "  int " + INSTR_DLENS_NAME + "[] = {\n    " + ",\n    ".join(instr_dlens) + "\n  };\n"
instr_ops_string = "  char " + INSTR_OPS_NAME + "[][" + str(OPS_MAX_LENGTH) + "] = {\n    " + ",\n    ".join(instr_ops) + "\n  };\n"

reg_toks_string = "  std::string " + REG_TOKS_NAME + "[] = {\n    \"" + "\",\n    \"".join(reg_toks) + "\"\n  };\n"
reg_addrs_string = "  int " + REG_ADDRS_NAME + "[] = {\n    " + ",\n    ".join(reg_addrs) + "\n  };\n"

fvs_toks_string = "  std::string " + FVS_TOKS_NAME + "[] = {\n    \"" + "\",\n    \"".join(fvs_toks) + "\"\n  };\n"
fvs_bits_string = "  int " + FVS_BITS_NAME + "[] = {\n    " + ",\n    ".join(fvs_bits) + "\n  };\n"
fvs_edits_string = "  bool " + FVS_EDITS_NAME + "[] = {\n    " + ",\n    ".join(fvs_edits) + "\n  };\n"

with open("../lang.hpp", "w") as f:
  f.write(header)

  f.write(instr_toks_string)
  f.write("\n")
  f.write(instr_vars_string)
  f.write("\n")
  f.write(instr_dlens_string)
  f.write("\n")
  f.write(instr_ops_string)

  f.write("\n")
  f.write(reg_toks_string)
  f.write("\n")
  f.write(reg_addrs_string)

  f.write("\n")
  f.write(fvs_toks_string)
  f.write("\n")
  f.write(fvs_bits_string)
  f.write("\n")
  f.write(fvs_edits_string)

  f.write(footer)
  f.close()
